/* 
 * Eneso Human Interface Device Library
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib.h>

#ifdef __MACH__

#include <CoreFoundation/CoreFoundation.h>
#include <Carbon/Carbon.h>
#include <IOKit/hid/IOHIDLib.h>
#include "ehid_macosx.h"

static ConnectionStatus	macosx_status	= DISCONNECTED;

static IOHIDManagerRef	manager	= NULL;
static IOHIDDeviceRef	dev		= NULL;


static void
device_matching_cb	(void* context, IOReturn result, void* sender,
		IOHIDDeviceRef device);
static void
device_removal_cb	(void* context, IOReturn result, void* sender,
		IOHIDDeviceRef device);
static void
input_report_cb		(void* context, IOReturn result, void* sender,
		IOHIDReportType type, uint32_t report_id, uint8_t* report, CFIndex length);



	void 
ehid_macosx_init (void)
{
	CFDictionaryRef	dict;
	CFNumberRef		number;

	/* Create a IO HID Manager */
	manager = IOHIDManagerCreate (kCFAllocatorDefault, kIOHIDOptionsTypeNone);
	require (manager, err);

	/* Create a device matching dictionary */
	dict = CFDictionaryCreateMutable (kCFAllocatorDefault, 0,
			&kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
	require (dict, err);

	/* VID */
	number = CFNumberCreate (kCFAllocatorDefault, kCFNumberIntType,
			&user_vid);
	CFDictionarySetValue (dict, CFSTR (kIOHIDVendorIDKey), number);
	CFRelease (number);

	/* PID */
	number = CFNumberCreate (kCFAllocatorDefault, kCFNumberIntType,
			&user_pid);
	CFDictionarySetValue (dict, CFSTR (kIOHIDProductIDKey), number);
	CFRelease (number);

	/* Set the matching dictionary */
	IOHIDManagerSetDeviceMatching (manager, dict);

	if (dict)
		CFRelease (dict);

	/* Register the device matching and removal callbacks */
	IOHIDManagerRegisterDeviceMatchingCallback (manager, device_matching_cb,
			NULL);
	IOHIDManagerRegisterDeviceRemovalCallback (manager, device_removal_cb,
			NULL);

	/* Schedule the HID manager to run with the current run loop */
	IOHIDManagerScheduleWithRunLoop (manager, CFRunLoopGetCurrent (),
			kCFRunLoopDefaultMode);

	/* Open the IO HID Manager */
	IOHIDManagerOpen (manager, kIOHIDOptionsTypeNone);

err:
	return;
}

	void 
ehid_macosx_terminate (void)
{
	if (!manager)
		return;

	/* Unregister the callbacks */
	IOHIDManagerRegisterDeviceMatchingCallback (manager, NULL, NULL);
	IOHIDManagerRegisterDeviceRemovalCallback (manager, NULL, NULL);

	/* Unschedule the HID manager */
	IOHIDManagerUnscheduleFromRunLoop (manager, CFRunLoopGetCurrent (),
			kCFRunLoopDefaultMode);

	/* Close the manager */
	IOHIDManagerClose (manager, 0);
}

	void 
ehid_macosx_send (const gchar* message)
{
	if (dev)
	{
		IOHIDDeviceSetReport (dev, kIOHIDReportTypeOutput, 0,
				message, user_output_length - 1);
	}
}

	ConnectionStatus
ehid_macosx_check (void)
{
	return macosx_status;
}

	static void
device_matching_cb (void* context, IOReturn result, void* sender,
		IOHIDDeviceRef device)
{
	CFTypeRef	ref;
	long		input_length;
	long		output_length;

	/* Get the input and output report length */
	ref = IOHIDDeviceGetProperty (device, CFSTR (kIOHIDMaxInputReportSizeKey));
	CFNumberGetValue ((CFNumberRef) ref, kCFNumberSInt32Type, &input_length);

	ref = IOHIDDeviceGetProperty (device, CFSTR (kIOHIDMaxOutputReportSizeKey));
	CFNumberGetValue ((CFNumberRef) ref, kCFNumberSInt32Type, &output_length);

	/* If they match, it is our device */
	if (((guint) input_length == user_input_length - 1) &&
			((guint) output_length == user_output_length - 1))
	{
		dev = device;
		macosx_status = CONNECTED;

		IOHIDDeviceRegisterInputReportCallback (device, read_buffer,
				user_input_length - 1, input_report_cb, NULL);

		check_connected_and_notify ();
	}
}

	static void
device_removal_cb (void* context, IOReturn result, void* sender,
		IOHIDDeviceRef device)
{
	if (device == dev)
	{
		dev = NULL;
		macosx_status = DISCONNECTED;

		check_connected_and_notify ();
	}
}

	static void
input_report_cb (void* context, IOReturn result, void* sender,
		IOHIDReportType type, uint32_t report_id, uint8_t* report, CFIndex length)
{
	if (user_message_cb)
		user_message_cb (read_buffer, length);
}

#endif /* __MACH__ */
